/**
Copyright (C) 2013 Bogdan Taranu
Permission is hereby granted, free of charge, to any person obtaining a copy of this 
software and associated documentation files (the "Software"), to deal in the Software 
without restriction, including without limitation the rights to use, copy, modify, 
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to 
permit persons to whom the Software is furnished to do so, subject to the following 
conditions:

The above copyright notice and this permission notice shall be included in all copies 
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
IN THE SOFTWARE.
 */
library html_tablesorter;

import "dart:html";
import "src/utils.dart";

/**
 * A basic tablesorter
 */
class TableSorter
{
  /**
   * The use DOM table
   */
  TableElement _table;
  /**
   * The original <th> (without links)
   */
  String _originalHeaders;
  /**
   * Used for selecting  
   */
  int _sortingKeyCellId;
  
  /**
   * Creates a new TableSorter object according to the ID of a table
   */
  TableSorter(String tableId)
  {
    _table = query(tableId);
    _originalHeaders = _table.tHead.innerHtml;
    _sortingKeyCellId = 0;    
  }  
  
  /**
   * Enables/Disables sorting for the selected table
   */
  void enableSorting(bool isEnabled)
  {
    if(isEnabled==true)
    {
      _table.tHead.rows[0].cells.forEach(this._addLink);  
    }
    else
    {
      _table.tHead.innerHtml = _originalHeaders;  
    }
  }
  
  /**
   * Returns the selected table
   */
  TableElement get table
  {
    return _table;
  }
  
  /**
   * Adds a link for <th> cells
   */
  void _addLink(TableCellElement cell)
  {
    AnchorElement anchor = new AnchorElement();
    anchor.href="#";
    anchor.text = cell.text;
    anchor.onClick.listen( (e) => this._sortTable(e) );
    cell.children.clear();
    cell.children.add(anchor);
  }
  
  /**
   * Used for comparing two table rows
   */
  int _compareRows(TableRowElement firstElement, 
                   TableRowElement secondElement)
  {
    int result = 0;
    String firstCell = firstElement.cells[_sortingKeyCellId]
                        .text;
    String secondCell = secondElement.cells[_sortingKeyCellId]
                        .text;
    int firstInt = tryParseInt(firstCell);
    int secondInt = tryParseInt(secondCell);
    if( ( firstInt==null ) || ( secondInt==null ) )
    {
      double firstDouble = tryParseDouble(firstCell);
      double secondDouble = tryParseDouble(secondCell);
      if( ( firstDouble == null ) || ( secondDouble == null ) )
      {
        result = firstCell.compareTo(secondCell);
      }
      else
      {
        result = firstDouble.compareTo(secondDouble);
      }
    }
    else
    {
      result = firstInt.compareTo(secondInt);
    }
    return result;
  }
  
  /**
   * Verifies if a list is already sorted.
   */
  bool _isAlreadySorted(List<TableRowElement> rows)
  {
    bool result = true;
    for(int i = 1; i < rows.length ; i++)
    {
      if( this._compareRows(rows[i-1], rows[i])>0 )
      {
        result = false;
        break;
      }
    }
    return result;
  }
  
  /**
   * Transforms a list of TableRowElement in HTML
   */
  String _getRowsNewHtml(List<TableRowElement> rows)
  {
    StringBuffer htmlBuffer = new StringBuffer();
    for(int i = 0; i<rows.length; i++)
    {
      htmlBuffer.write(rows[i].outerHtml);      
    }
    String result = htmlBuffer.toString();
    return result;
  }
  
  /**
   * Sort the table according to the selected cell key
   */
  void _sortTable(Event e)
  {
    _sortingKeyCellId= ( (e.target as Element)
                       .parent as TableCellElement )
                       .cellIndex;
    List<TableRowElement> rowsList;
    if(_isAlreadySorted(_table.tBodies[0].rows)==false)
    {
      rowsList = _table.tBodies[0].rows.toList(growable: false);
      rowsList.sort( (a,b) => this._compareRows( a, b ) );        
    }
    else
    {
      rowsList = _table.tBodies[0].rows.reversed.toList(growable: false);
    }
    _table.tBodies[0].innerHtml = "";
    _table.tBodies[0].innerHtml = this._getRowsNewHtml(rowsList);  
  }  
  
}